/**************************************************************
VERMIS
Copyright (C) 2003  Antoine Cellerier
e-mail : antoinecellerier@yahoo.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See vermis.txt for more info on game use
**************************************************************/

// C Source File
// Created 29/03/2003; 17:34:34

//COMPILE ONE CALC AT A TIME (0 : TI-89 or 1 : TI-92+ or V200)//
#if 0
	#define USE_TI92PLUS          // Compile for TI-92 Plus
	#define USE_V200              // Compile for V200
#else
	#define USE_TI89              // Compile for TI-89
#endif

//LANGUGE SELECTION//
/*	0 : english (default)
	1 : french
	if you wish you can make your own language files
	to do that copy the lang_en.h file and change the
	strings defined in it.
	be carefull to keep the %d symbols relative to
	display of variables from the game.
	please send them to me if u do so
	thanks*/
#define LANGUAGE 0

#if LANGUAGE==1
	#include "lang_fr.h"

//#elif LANGUAGE==?
//	#include "lang_??.h"

#else
	#include "lang_en.h"
#endif

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#define VER "v0.2c"
#define GAME_NAME "Vermis"
#define VER2 "Vermis v0.2c"
#define EMAIL "antoinecellerier@yahoo.com"

#define VERMIS_NUM 1
#define HUMAN_NUM 1

#define HIGH_SCORE 0
#define XMAX ( LCD_WIDTH - 1 )
#define YMAX ( LCD_HEIGHT - 7 )
#define INIT_LEN 9
#define GROWTH 20
#define APPLE_NUM 10
#define BLOCK_NUM 2
#define APPLE_SIZE 3
#define BLOCK_SIZE 4
#define TIMER 20
#define STATS_ON 1
#define COMMAND_MODE 1 // 0 : 2 key  -  1 : 4 key
#define COMMAND_MODE2 1 // 0 : 2 key  -  1 : 4 key

#define STATS_Y ( LCD_HEIGHT - 5 )

#define XMAX_MIN 20
#define YMAX_MIN 20
#define XMAX_MAX ( LCD_WIDTH - 1 )
#define YMAX_MAX ( LCD_HEIGHT - 7 )
#define INIT_LEN_MAX 2047
#define GROWTH_MAX 2047
#define APPLE_NUM_MAX 254
#define BLOCK_NUM_MAX 254
#define APPLE_SIZE_MIN 3
#define BLOCK_SIZE_MIN 1
#define APPLE_SIZE_MAX 30
#define BLOCK_SIZE_MAX 30
#define TIMER_MAX 400
#define VERMIS_NUM_MAX 100

//TYPEDEFS
typedef struct position {
	unsigned char x;
	unsigned char y;	
}position;

typedef struct setting {
	unsigned int xmax;
	unsigned int ymax;
	unsigned int init_len;
	unsigned int growth;
	unsigned int apple_num;
	unsigned int block_num;
	unsigned int apple_size;
	unsigned int block_size;
	unsigned int timer;
	BOOL stats_on;
	BOOL command_mode;
	BOOL command_mode2;
	unsigned int vermis_num;
	unsigned int human_num;
}setting;



//MENU FUNCTIONS
void load_set(setting set);

void save_set(setting *set);

void read_key(unsigned short *c, unsigned short *r);

void p1_key_change(void);

void p2_key_change(void);

void menu1(void);

void menu2(void);

//IN GAME FUNCTIONS
BOOL test_block(position pos);

BOOL test_apple(position pos);

int is_apple(position test_pos);

void draw_apple(unsigned int i);

void new_apple(unsigned int i);

void new_block(void);

//OTHER FUNCTIONS
void reset_autoints(void);

volatile int counter=0;


//INTERRUPT HANDLERS

//395 Hz
INT_HANDLER OldInt1 = NULL;
DEFINE_INT_HANDLER (MyInt1){
	if(counter){
		counter--;
	}
}

//19 Hz
INT_HANDLER OldInt5 = NULL;
DEFINE_INT_HANDLER (MyInt5){
	;
}

//[ON] key
INT_HANDLER OldInt6 = NULL;
DEFINE_INT_HANDLER (MyInt6){
	if(_keytest(RR_2ND)){
		reset_autoints();
		off();
		SetIntVec (AUTO_INT_1, MyInt1);
		SetIntVec (AUTO_INT_5, MyInt5);
		SetIntVec (AUTO_INT_6, MyInt6);
	}	
}

void reset_autoints(void){
	SetIntVec (AUTO_INT_1, OldInt1);
	SetIntVec (AUTO_INT_5, OldInt5);
	SetIntVec (AUTO_INT_6, OldInt6);
}

//GLOBAL VAR DECLARATIONS

#if CALCULATOR   // TI-92+ and V200 default keys
	
	//player 1 default keys used in _keytest
	//up
	unsigned short u_p1_r = 0;
	unsigned short u_p1_c = 5;
	//right
	unsigned short r_p1_r = 0;
	unsigned short r_p1_c = 6;
	//down
	unsigned short d_p1_r = 0;
	unsigned short d_p1_c = 7;
	//left
	unsigned short l_p1_r = 0;
	unsigned short l_p1_c = 4;
	
	//player 2 default keys used in _keytest
	//key up : y
	unsigned short u_p2_r = 1;
	unsigned short u_p2_c = 3;
	//key right: )
	unsigned short r_p2_r = 1;
	unsigned short r_p2_c = 2;
	//key down : 7
	unsigned short d_p2_r = 1;
	unsigned short d_p2_c = 1;
	//key left : =
	unsigned short l_p2_r = 9;
	unsigned short l_p2_c = 2;

#else  // TI-89 default keys
	//player 1 default keys used in _keytest
	//up
	unsigned short u_p1_r = 0;
	unsigned short u_p1_c = 0;
	//right
	unsigned short r_p1_r = 0;
	unsigned short r_p1_c = 3;
	//down
	unsigned short d_p1_r = 0;
	unsigned short d_p1_c = 2;
	//left
	unsigned short l_p1_r = 0;
	unsigned short l_p1_c = 1;
	
	//player 2 default keys used in _keytest
	//key up : y
	unsigned short u_p2_r = 4;
	unsigned short u_p2_c = 5;
	//key right: )
	unsigned short r_p2_r = 3;
	unsigned short r_p2_c = 4;
	//key down : 7
	unsigned short d_p2_r = 4;
	unsigned short d_p2_c = 4;
	//key left : =
	unsigned short l_p2_r = 5;
	unsigned short l_p2_c = 4;

#endif



unsigned int high_score = HIGH_SCORE;

unsigned int xmax = XMAX;
unsigned int ymax = YMAX;

unsigned int init_len = INIT_LEN;
unsigned int growth = GROWTH;

unsigned int apple_num = APPLE_NUM;
unsigned int block_num = BLOCK_NUM;

unsigned int apple_size = APPLE_SIZE;
unsigned int block_size = BLOCK_SIZE;

unsigned int timer = TIMER;

BOOL stats_on = STATS_ON;

BOOL command_mode = COMMAND_MODE;
BOOL command_mode2 = COMMAND_MODE2;

unsigned int vermis_num = VERMIS_NUM; //includes human players
unsigned int human_num = HUMAN_NUM;

position **pl;
position *buffer;

position *apple_pos;

#include "menus.h"

//FUNCTIONS

BOOL test_block(position pos){
	unsigned int i,j;
	for(i=0;i<block_size;i++){
		for(j=0;j<block_size;j++){
			if(GetPix((short)pos.x+i,(short)pos.y+j)) return 1;
		}
	}
	return 0;
}

BOOL test_apple(position pos){
	unsigned int i;
	for(i=0;i<apple_size;i++){		
		if(GetPix((short)pos.x+i,(short)pos.y)) return 1;
		if(GetPix((short)pos.x+i,(short)pos.y+apple_size-1)) return 1;
		if(GetPix((short)pos.x,(short)pos.y+i)) return 1;
		if(GetPix((short)pos.x+apple_size-1,(short)pos.y+i)) return 1;
	}
	return 0;
}

int is_apple(position test_pos){
	unsigned int i;
	for(i=0;i<apple_num;i++){
		if (apple_pos[i].x <= test_pos.x && apple_pos[i].x + apple_size - 1 >= test_pos.x &&
		    apple_pos[i].y <= test_pos.y && apple_pos[i].y + apple_size - 1 >= test_pos.y) {
		    	return i;
		    }
	}
	return 255;
}


void draw_apple(unsigned int i){
	DrawLine((short)apple_pos[i].x,(short)apple_pos[i].y,(short)apple_pos[i].x,(short)apple_pos[i].y+apple_size-1,A_XOR);
	DrawLine((short)apple_pos[i].x+apple_size-1,(short)apple_pos[i].y,(short)apple_pos[i].x+apple_size-1,(short)apple_pos[i].y+apple_size-1,A_XOR);
	DrawLine((short)apple_pos[i].x+1,(short)apple_pos[i].y+apple_size-1,(short)apple_pos[i].x+apple_size-2,(short)apple_pos[i].y+apple_size-1,A_XOR);
	DrawLine((short)apple_pos[i].x+1,(short)apple_pos[i].y,(short)apple_pos[i].x+apple_size-2,(short)apple_pos[i].y,A_XOR);
	return;
}

void new_apple(unsigned int i){
	position apple;
	int count=0;
	do {
		apple.x = random(xmax-apple_size+1);
		apple.y = random(ymax-apple_size+1);
		if (count++ > 255) return;
	} while (test_apple(apple));
	apple_pos[i] = apple;
	draw_apple(i);
	
	return;
}

void new_block(void) {
	position block;
	unsigned int count=0;
	do {
		block.x = random(xmax-block_size+1);
		block.y = random(ymax-block_size+1);
		if (count++ > 255) return;
	} while (test_block(block));
	
	for(count=0;count<block_size;count++){
		DrawLine((short)block.x,(short)block.y+count,(short)block.x+block_size-1,(short)block.y+count,A_XOR);
	}
	
	return;
}


// Main Function
void _main(void)
{
	menu1();
	
	if(vermis_num < 1){
		vermis_num = 1;
	}
	
	char* str1,* str2,* str3,* str4;
	
	str1 = (char *)malloc(10 * sizeof(char));
	str2 = (char *)malloc(10 * sizeof(char));
	str3 = (char *)malloc(10 * sizeof(char));
	str4 = (char *)malloc(10 * sizeof(char));
	
	sprintf(str1,"-");
	sprintf(str2,"-");
	sprintf(str3,TIMER_NONE);
	sprintf(str4,"0");
	
	clrscr();
	
	FontSetSys(F_4x6);
	
	randomize();
	
	DrawLine(0,0,0,ymax,A_NORMAL);
	DrawLine(0,0,xmax,0,A_NORMAL);
	DrawLine(xmax,0,xmax,ymax,A_NORMAL);
	DrawLine(0,ymax,xmax,ymax,A_NORMAL);
	
	
	#if CALCULATOR
		DrawStr(180,STATS_Y,VER2 ,A_NORMAL);
	#endif
	
	if(stats_on){
		DrawStr(0,STATS_Y,P1_TEXT,A_NORMAL);
		DrawStr(15,STATS_Y,str1,A_NORMAL);
		
		DrawStr(35,STATS_Y,P2_TEXT,A_NORMAL);
		DrawStr(50,STATS_Y,str2,A_NORMAL);
		
		DrawStr(70,STATS_Y,NUM_VERMIS_TEXT,A_NORMAL);
		DrawStr(90,STATS_Y,str4,A_NORMAL);
		
		DrawStr(110,STATS_Y,TIMER_TEXT,A_NORMAL);
		DrawStr(140,STATS_Y,str3,A_NORMAL);
	} else {
		DrawStr(0,STATS_Y,VER2,A_NORMAL);
	}
	
	
	unsigned int i,j,vn;
	position temp_pos;
	
	unsigned int *g;
	if((g = (unsigned int *)malloc(vermis_num * sizeof(unsigned int)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	
	BOOL *the_fun_goes_on;
	if((the_fun_goes_on = (BOOL *)malloc(vermis_num * sizeof(BOOL)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	
	unsigned char the_fun_goes_on_for_everyone=1;
	
	BOOL *is_ia;
	if((is_ia = (BOOL *)malloc(vermis_num * sizeof(BOOL)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	
	unsigned int *num;
	if((num = (unsigned int *)malloc(vermis_num * sizeof(unsigned int)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	
	char *xm;
	char *ym;
	if((xm = (char *)malloc(vermis_num * sizeof(char)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	if((ym = (char *)malloc(vermis_num * sizeof(char)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	
	if((pl = (position **)malloc(vermis_num * sizeof(position *)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
	
	for(vn=0;vn<vermis_num;vn++){
		num[vn]=1;
		if((pl[vn] = (position *)malloc(num[vn] * sizeof(position)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);ngetchx();return;}
		pl[vn][0].x = random(xmax - 15) + 7;
		pl[vn][0].y = random(ymax - 15) + 7;
		xm[vn] = 1;
		ym[vn] = 0;
		g[vn] = init_len;
		the_fun_goes_on[vn]=1;
		is_ia[vn]=1;
	}
	
	/*
	players 1 and 2 are local
	players 3 and 4 are remote (not available yet)
	*/
	if(human_num){
		for(vn=0;vn<human_num;vn++){
			is_ia[vn]=0;
			switch(vn){
				case 0:
					pl[vn][0].x =1;
					pl[vn][0].y =1;
					xm[vn] = 1;
					ym[vn] = 0;
					DrawLine(1,1,10,1,A_XOR); // prevent vermis from starting in a block
					break;
				
				case 1:
					pl[vn][0].x =1;
					pl[vn][0].y =ymax - 1;
					xm[vn] = 1;
					ym[vn] = 0;
					DrawLine(1,ymax - 1,10,ymax - 1,A_XOR); // prevent vermis from starting in a block
					break;
				
				case 2:
					pl[vn][0].x =xmax - 1;
					pl[vn][0].y =ymax - 1;
					xm[vn] = -1;
					ym[vn] = 0;
					DrawLine(xmax - 10,ymax - 1,xmax - 1,ymax - 1,A_XOR); // prevent vermis from starting in a block
					break;
				
				case 3:
					pl[vn][0].x =xmax - 1;
					pl[vn][0].y =1;
					xm[vn] = -1;
					ym[vn] = 0;
					DrawLine(xmax - 10,1,xmax - 1,1,A_XOR); // prevent vermis from starting in a block
					break;
				

			}
		}
	}
	
	if((apple_pos = (position *)malloc(apple_num * sizeof(position)))==NULL) {printf("error line %d",__LINE__);ngetchx();return;}
	
	for(i=0;i<apple_num;i++){
		apple_pos[i] = pl[0][0];
	}
	
	for(i=0;i<apple_num;i++){
		new_apple(i);
		if(block_num!=0) {for(j=0;j<block_num;j++) new_block();}
	}
	
	if(human_num){
		for(vn=0;vn<human_num;vn++){
			switch(vn){
				case 0:
					DrawLine(1,1,10,1,A_XOR); // prevent vermis from starting in a block
					break;
				
				case 1:
					DrawLine(1,ymax - 1,10,ymax - 1,A_XOR); // prevent vermis from starting in a block
					break;
				
				case 2:
					DrawLine(xmax - 10,ymax - 1,xmax - 1,ymax - 1,A_XOR); // prevent vermis from starting in a block
					break;
				
				case 3:
					DrawLine(xmax - 10,1,xmax - 1,1,A_XOR); // prevent vermis from starting in a block
					break;
			}
		}
	}
	
	
	
	OldInt1 = GetIntVec (AUTO_INT_1);
	SetIntVec (AUTO_INT_1, MyInt1);
	
	OldInt5 = GetIntVec (AUTO_INT_5);
	SetIntVec (AUTO_INT_5, MyInt5);
	
	OldInt6 = GetIntVec (AUTO_INT_6);
	SetIntVec (AUTO_INT_6, MyInt6);
	
	counter = timer;
	
	while (the_fun_goes_on_for_everyone) {
		
		the_fun_goes_on_for_everyone = 0;
		
		for(vn=0;vn<vermis_num;vn++){
			if(the_fun_goes_on[vn]){
				
				the_fun_goes_on_for_everyone += the_fun_goes_on[vn];
				
				/* "artificial vermis intelligence"
				v0.2 : just keep out of deadly obstacles - random movement
				*/
				if(is_ia[vn]){ 
					if(!random(20)){
						if(random(2)){
							xm[vn] = xm[vn] + ym[vn];
							ym[vn] = xm[vn] - ym[vn];
							xm[vn] = ym[vn] - xm[vn];	
						} else {
							xm[vn] = xm[vn] + ym[vn];
							ym[vn] = ym[vn] - xm[vn];
							xm[vn] = xm[vn] + ym[vn];
						}
					}
					temp_pos.x = pl[vn][num[vn]-1].x + xm[vn];
					temp_pos.y = pl[vn][num[vn]-1].y + ym[vn];
					if(GetPix((short)temp_pos.x,(short)temp_pos.y)
					&& is_apple(temp_pos)==255){
						if(random(2)){
							xm[vn] = xm[vn] + ym[vn];
							ym[vn] = xm[vn] - ym[vn];
							xm[vn] = ym[vn] - xm[vn];
							temp_pos.x = pl[vn][num[vn]-1].x + xm[vn];
							temp_pos.y = pl[vn][num[vn]-1].y + ym[vn];
							if(GetPix((short)temp_pos.x,(short)temp_pos.y)
							&& is_apple(temp_pos)==255){
								xm[vn] = -xm[vn];
								ym[vn] = -ym[vn];
							}	
						} else {
							xm[vn] = xm[vn] + ym[vn];
							ym[vn] = ym[vn] - xm[vn];
							xm[vn] = xm[vn] + ym[vn];
							temp_pos.x = pl[vn][num[vn]-1].x + xm[vn];
							temp_pos.y = pl[vn][num[vn]-1].y + ym[vn];
							if(GetPix((short)temp_pos.x,(short)temp_pos.y)
							&& is_apple(temp_pos)==255){
								xm[vn] = -xm[vn];
								ym[vn] = -ym[vn];
							}
						}	
					}
				}
				
				
				
				
				buffer = pl[vn];
				
				if (g[vn]>0) {
					g[vn]--;
					num[vn]++;
					if((pl[vn] = (position *)malloc(num[vn] * sizeof(position)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);reset_autoints();ngetchx();return;}
					memmove(pl[vn],buffer,(num[vn]-1)*sizeof(position));
					
				} else {
					if((pl[vn] = (position *)malloc(num[vn] * sizeof(position)))==NULL) {printf("error line %d\nnot enough memory",__LINE__);reset_autoints();ngetchx();return;}
					memmove(pl[vn],&buffer[1],(num[vn]-1)*sizeof(position));
					DrawPix((short)pl[vn][0].x,(short)pl[vn][0].y,A_REVERSE);
				}
				
				free(buffer);
				pl[vn][num[vn]-1].x = pl[vn][num[vn]-2].x + xm[vn];
				pl[vn][num[vn]-1].y = pl[vn][num[vn]-2].y + ym[vn];
				
				DrawPix((short)pl[vn][num[vn]-1].x,(short)pl[vn][num[vn]-1].y,A_XOR);
				
				if(!GetPix((short)pl[vn][num[vn]-1].x,(short)pl[vn][num[vn]-1].y)){
					the_fun_goes_on[vn] = 0;
					
					if((i=is_apple(pl[vn][num[vn]-1]))!=255){
						g[vn] += growth;
						draw_apple(i);
						new_apple(i);
						if(block_num!=0) {for(j=0;j<block_num;j++) new_block();}
						the_fun_goes_on[vn] = 1;
					} else {
						DrawPix((short)pl[vn][num[vn]-1].x,(short)pl[vn][num[vn]-1].y,A_XOR);
					}
				}
			}
		}
		
		if (stats_on){
			if (human_num>0) {
				DrawStr(15,STATS_Y,str1,A_XOR);
				sprintf(str1,"%d",num[0]);
				DrawStr(15,STATS_Y,str1,A_XOR);
				if (human_num>1) {
					DrawStr(50,STATS_Y,str2,A_XOR);
					sprintf(str2,"%d",num[1]);
					DrawStr(50,STATS_Y,str2,A_XOR);
				}
			}
			DrawStr(90,STATS_Y,str4,A_XOR);
			sprintf(str4,"%d",the_fun_goes_on_for_everyone);
			DrawStr(90,STATS_Y,str4,A_XOR);
			DrawStr(140,STATS_Y,str3,A_XOR);
			if(counter){
				sprintf(str3,"ok");	
			} else {
				sprintf(str3,"exp");	
			}	
			DrawStr(140,STATS_Y,str3,A_XOR);
		}
		
		while (counter!=0);
		counter=timer;
		
		/* player 1 input */
		if(human_num>0){
			if(_keytest(u_p1_r,u_p1_c)){
				if(command_mode){
					if (ym[0]==0) {
						xm[0] = 0;
						ym[0] = -1;	
					}
				}
			}
			
			if(_keytest(r_p1_r,r_p1_c)){
				if(command_mode){
					if (xm[0]==0) {
						xm[0] = 1;
						ym[0] = 0;	
					}
				} else {
					xm[0] = xm[0] + ym[0];
					ym[0] = xm[0] - ym[0];
					xm[0] = ym[0] - xm[0];
				}
			}
				
			
			if(_keytest(d_p1_r,d_p1_c)){
				if(command_mode){
					if (ym[0]==0) {
						xm[0] = 0;
						ym[0] = 1;
					}
				}
			}
			
			if(_keytest(l_p1_r,l_p1_c)){
				if(command_mode){
					if (xm[0]==0) {
						xm[0] = -1;
						ym[0] = 0;	
					}
				} else {
					xm[0] = xm[0] + ym[0];
					ym[0] = ym[0] - xm[0];
					xm[0] = xm[0] + ym[0];
				}
			}
		}
		
		/* player 2 input */
		if(human_num>1){
			if(_keytest(u_p2_r,u_p2_c)){
				if(command_mode2){
					if (ym[1]==0) {
						xm[1] = 0;
						ym[1] = -1;	
					}
				}
			}
			
			if(_keytest(r_p2_r,r_p2_c)){
				if(command_mode2){
					if (xm[1]==0) {
						xm[1] = 1;
						ym[1] = 0;	
					}
				} else {
					xm[1] = xm[1] + ym[1];
					ym[1] = xm[1] - ym[1];
					xm[1] = ym[1] - xm[1];
				}
			}
			
			if(_keytest(d_p2_r,d_p2_c)){
				if(command_mode2){
					if (ym[1]==0) {
						xm[1] = 0;
						ym[1] = 1;
					}
				}
			}
			
			if(_keytest(l_p2_r,l_p2_c)){
				if(command_mode2){
					if (xm[1]==0) {
						xm[1] = -1;
						ym[1] = 0;	
					}
				} else {
					xm[1] = xm[1] + ym[1];
					ym[1] = ym[1] - xm[1];
					xm[1] = xm[1] + ym[1];
				}
			}
		}
		
		//pause
		if(_keytest(RR_APPS)){
			DrawStr(140,STATS_Y,str3,A_XOR);
			DrawStr(140,STATS_Y,"PAUSE",A_XOR);	
			while(_keytest(RR_APPS));
			while(!_keytest(RR_APPS));
			while(_keytest(RR_APPS));
			DrawStr(140,STATS_Y,"PAUSE",A_XOR);
			DrawStr(140,STATS_Y,str3,A_XOR);			
		}
		
		//quit
		if(_keytest(RR_ESC))
			the_fun_goes_on_for_everyone = 0;
		
		/* end of input */
		
	}
	
	counter=300;
	while(counter){
		if(_keytest(RR_ESC)) break;
	};

	
	for(vn=0;vn<vermis_num;vn++){
		free(pl[vn]);
	}
	free(pl);
	free(g);
	free(xm);
	free(ym);
	free(the_fun_goes_on);
	free(is_ia);
	free(apple_pos);
	
	free(str1);
	free(str2);
	free(str3);
	free(str4);
	

	
	
	i=0;
	j=0;
	for(vn=0;vn<vermis_num;vn++){
		if(num[vn]>num[i]) i = vn;
		if(num[vn]<num[j]) j = vn;
	}
	
	if(num[i]>high_score) high_score = num[i];
	
	clrscr();
	
	
	FontSetSys( F_4x6);
	
	printf(GAME_OVER);
	
	if(vermis_num>1){
		printf(WAS_LONGEST,i+1,num[i]);
		printf(WAS_SHORTEST,j+1,num[j]);
	}
	if(human_num>0) printf(P1_LENGTH,num[0]);
	if(human_num>1) printf(P2_LENGTH,num[1]);
	if(num[i]==high_score){
		printf(BEAT_HIGHSCORE,i+1,high_score);
	} else {
		printf(DIDNT_REACH_HIGHSCORE,high_score);
	}
	
	free(num);
	
	printf("\n\n");
	FontSetSys(F_8x10);
	printf("- %s -\n",GAME_NAME);
	FontSetSys(F_4x6);
	printf(BROUGH_TO_YOU_BY);
	printf("%s\n",EMAIL);
	printf("%s %s %s\n",VER,__DATE__,__TIME__);
	printf(CHECK_FOR_UPDATES);
	
	while(_rowread(0)==0);
	while(_rowread(0));
	
	reset_autoints();
	
	return;
}
